GSTR_APPFOLDER = 1
GSTR_CONFOLDER = 2
GSTR_LANFOLDER = 3
GSTR_TEMFOLDER = 4
GSTR_CE_FILE   = 5
GSTR_CONF_FILE = 6
GSTR_LOC_FILE  = 7
GSTR_SSET_FILE = 8
GSTR_LOCX_FILE  = 9
GSTR_CEX_FILE   = 10
GSTR_CONFX_FILE = 11
GSTR_TZ_FILE    = 12
GSTR_COUNTRY_FILE = 13
GSTR_TEXT_FILE = 14
GSTR_TIPS_FILE = 15
GSTR_HELP_FILE = 16
