TITHI_KRSNA_PRATIPAT = 0
TITHI_KRSNA_DVITIYA = 1
TITHI_KRSNA_TRITIYA = 2
TITHI_KRSNA_CATURTI = 3
TITHI_KRSNA_PANCAMI = 4
TITHI_KRSNA_SASTI = 5
TITHI_KRSNA_SAPTAMI = 6
TITHI_KRSNA_ASTAMI = 7
TITHI_KRSNA_NAVAMI = 8
TITHI_KRSNA_DASAMI = 9
TITHI_KRSNA_EKADASI = 10
TITHI_KRSNA_DVADASI = 11
TITHI_KRSNA_TRAYODASI = 12
TITHI_KRSNA_CATURDASI = 13
TITHI_AMAVASYA = 14
TITHI_GAURA_PRATIPAT = 15
TITHI_GAURA_DVITIYA = 16
TITHI_GAURA_TRITIYA = 17
TITHI_GAURA_CATURTI = 18
TITHI_GAURA_PANCAMI = 19
TITHI_GAURA_SASTI = 20
TITHI_GAURA_SAPTAMI = 21
TITHI_GAURA_ASTAMI = 22
TITHI_GAURA_NAVAMI = 23
TITHI_GAURA_DASAMI = 24
TITHI_GAURA_EKADASI = 25
TITHI_GAURA_DVADASI = 26
TITHI_GAURA_TRAYODASI = 27
TITHI_GAURA_CATURDASI = 28
TITHI_PURNIMA = 29
