CCTYPE_DATE = 1
CCTYPE_S_ARUN = 10
CCTYPE_S_RISE = 11
CCTYPE_S_NOON = 12
CCTYPE_S_SET =  13
CCTYPE_TITHI =  20
CCTYPE_NAKS  =  21
CCTYPE_SANK  =  22
CCTYPE_CONJ  =  23
CCTYPE_YOGA  = 24
CCTYPE_KALA_START=  30
CCTYPE_KALA_END  =  31
CCTYPE_M_RISE = 41
CCTYPE_M_SET  = 42
CCTYPE_M_RASI = 45
CCTYPE_ASCENDENT = 50
