SPEC_RETURNRATHA    =  3
SPEC_HERAPANCAMI    =  4
SPEC_GUNDICAMARJANA =  5
SPEC_GOVARDHANPUJA  =  6
SPEC_VAMANADVADASI  =  7
SPEC_VARAHADVADASI  =  8
SPEC_RAMANAVAMI     =  9
SPEC_JANMASTAMI     = 10
SPEC_RATHAYATRA     = 11
SPEC_GAURAPURNIMA   = 12
SPEC_NANDAUTSAVA    = 13
SPEC_MISRAFESTIVAL  = 14
SPEC_PRABHAPP       = 15
