MESHA_SANKRANTI = 0
VRSABHA_SANKRANTI = 1
MITHUNA_SANKRANTI = 2
KATAKA_SANKRANTI = 3
SIMHA_SANKRANTI = 4
KANYA_SANKRANTI = 5
TULA_SANKRANTI = 6
VRSCIKA_SANKRANTI = 7
DHANUS_SANKRANTI = 8
MAKARA_SANKRANTI = 9
KUMBHA_SANKRANTI = 10
MINA_SANKRANTI = 11
